@@ -9,7 +9,7 @@ class Point:
         The prescribed mesh element size at this point.
     """
 
-    dimension = 0
+    dim = 0
 
     def __init__(self, env, x, mesh_size=None):
         if len(x) == 2:
@@ -21,7 +21,8 @@ class Point:
         if mesh_size is not None:
             args.append(mesh_size)
         self._ID = env.addPoint(*args)
-        self.dim_tags = [(0, self._ID)]
+        self.dim_tag = (0, self._ID)
+        self.dim_tags = [self.dim_tag]
 
     def __repr__(self):
         X = ", ".join(str(x) for x in self.x)
