@@ -4,38 +4,34 @@ import numpy
 from .. import common
 
 
+class Circle:
+    def __init__(
+        self,
+        x0,
+        radius,
+        R,
+        compound,
+        num_sections,
+        holes,
+        curve_loop,
+        plane_surface,
+        mesh_size=None,
+    ):
+        self.x0 = x0
+        self.radius = radius
+        self.mesh_size = mesh_size
+        self.R = R
+        self.compound = compound
+        self.num_sections = num_sections
+        self.holes = holes
+        self.curve_loop = curve_loop
+        self.plane_surface = plane_surface
+
+
 class Geometry(common.CommonGeometry):
     def __init__(self):
         super().__init__(gmsh.model.geo)
 
-    # All of the add_* method below could be replaced by
-    #
-    #   def add(self, entity):
-    #       self._GMSH_CODE.append(entity.code)
-    #       return entity
-    #
-    # to be used like
-    #
-    #    geom.add(pg.Circle(...))
-    #
-    # However, this would break backwards compatibility and perhaps encourage
-    # users to do
-    #
-    #    c = pg.Circle(...)
-    #    # ... use c
-    #
-    # in which case the circle code never gets added to geom.
-
-    def set_transfinite_curve(self, curve, num_nodes, mesh_type, coeff):
-        assert mesh_type in ["Progression", "Bulk"]
-        self._TRANSFINITE_CURVE_QUEUE.append((curve._ID, num_nodes, mesh_type, coeff))
-
-    def set_transfinite_surface(self, surface, arrangement, corner_tags):
-        self._TRANSFINITE_SURFACE_QUEUE.append((surface._ID, arrangement, corner_tags))
-
-    def set_recombined_surfaces(self, surfaces):
-        self._RECOMBINE_ENTITIES += [s.dim_tags[0] for s in surfaces]
-
     def add_circle(
         self,
         x0,
@@ -100,29 +96,6 @@ class Geometry(common.CommonGeometry):
         else:
             plane_surface = None
 
-        class Circle:
-            def __init__(
-                self,
-                x0,
-                radius,
-                R,
-                compound,
-                num_sections,
-                holes,
-                curve_loop,
-                plane_surface,
-                mesh_size=None,
-            ):
-                self.x0 = x0
-                self.radius = radius
-                self.mesh_size = mesh_size
-                self.R = R
-                self.compound = compound
-                self.num_sections = num_sections
-                self.holes = holes
-                self.curve_loop = curve_loop
-                self.plane_surface = plane_surface
-
         return Circle(
             x0,
             radius,
@@ -226,7 +199,7 @@ class Geometry(common.CommonGeometry):
         volume = self.add_volume(surface_loop, holes) if with_volume else None
 
         class Ellipsoid:
-            dimension = 3
+            dim = 3
 
             def __init__(self, x0, radii, surface_loop, volume, mesh_size=None):
                 self.x0 = x0
